
function GTR_OnLoad(self)
	GTRWatchForMailInboxUpdate = 0;
	-- Note: need to add guild joining/leaving 
	-- money making events
	self:RegisterEvent("CHAT_MSG_MONEY");
	self:RegisterEvent("CHAT_MSG_SYSTEM");
	self:RegisterEvent("MERCHANT_SHOW");
	self:RegisterEvent("MERCHANT_CLOSED");
	-- Tax notification events
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("GUILD_ROSTER_UPDATE");
	self:RegisterEvent("GUILD_MOTD");
	self:RegisterEvent("CHAT_MSG_GUILD");
	self:RegisterEvent("CHAT_MSG_WHISPER");
	-- Mail -- tax auction money
	-- Guild bank access
	self:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED");
	-- other required events
	self:RegisterEvent("ADDON_LOADED");
	-- slash command
	SLASH_GTR1 = "/GTR";
	SlashCmdList["GTR"] = function(msg, editBox)
		if msg == "reset" then
			print("|cffFF7C0AGTR|r: Tax due reset!");
			GTRTaxDue = 0;
		elseif msg == "quiet" then
			GTRQuietMode = 1;
			print("|cffFF7C0AGTR|r: Quiet mode enabled.");
		elseif msg == "verbose" then
			GTRQuietMode = 0;
			print("|cffFF7C0AGTR|r: Verbose mode enabled.");
		elseif msg == "help" then
			print("|cffFF7C0AGTR|r: Version: "..GetAddOnMetadata("GTR","Version")); 
			print("To use, all guild members that will be taxed must have the add-on installed.");
			print("Set the tax rate by including in the Guild Info text a string in the format |cffFF0000tax = 10%|r (or whetever percentage you want)");
			print("Tax rates can be overriden for individual members by including a tax = xx% in the officer notes for the guild member (where xx is the tax percentage)");
			print("Tax rate defaults to 10% if not specified in the guild info/officer notes.");
			print("Income from loot, quest rewards & auction house will be taxed. Individual player transactions are not taxed. Any tax due will be automatically deposited (assuming the member has funds) next time the member opens the guild bank.");
			print("Slash commands usable by members:");
			print("|cffFF7C0A/GTR help|r -- prints this message");
			print("|cffFF7C0A/GTR reset|r -- reset tax due");
			print("|cffFF7C0A/GTR quiet|r -- turn on quiet(ish) mode, user won't get notified when amounts are taxed");
			print("|cffFF7C0A/GTR verbose|r -- turn on verbose mode (default), user will be notified of amounts taxed");
			print("|cffFF7C0A/GTR|r -- display version, current tax due and total tax paid");
			print("Officer commands:");
			print("Officers can type the commands below in the guild chat or in a private tell to a guild member. The addon on each member's computer will reply automatically.");
			print("|cffFF7C0A!GTRaudit|r -- audit the installation of GTR. If GTR is installed, each player will automatically whisper back with a message confirming the addon is installed, the current tax due and tax paid to date.");
			print("|cffFF7C0A!GTRguild|r -- like !GTRaudit, but replies will be sent to guild chat instead of being whispered back privately.");
			print("|cffFF7C0A!GTRupdate|r -- after updating the tax rate in the Guild Info/Officer Notes, force the addon to load the new tax rate.");
			print("The addon will ignore !GTR*** messages unless the player who sent the message is rank 5 or higher in the guild hierarchy (typically an officer).");
		else
			print("|cffFF7C0AGTR|r: Version: "..GetAddOnMetadata("GTR","Version")); 
			print(format("|cffFF7C0AGTR|r: Current Tax Rate: %d%%", GTRTaxPercentage)); 
			print("|cffFF7C0AGTR|r: Current Tax Due: "..GetCoinText(GTRTaxDue)); 
			print("|cffFF7C0AGTR|r: Total Tax Paid: "..GetCoinText(GTRTaxToDate));
		end
	end
	local origTakeInboxMoney = TakeInboxMoney;
	TakeInboxMoney = function(...)
		local index = ...;
		local invoiceType, itemName, playerName, bid, buyout, deposit, consignment = GetInboxInvoiceInfo(index);
		if invoiceType == "seller" then
			local packageIcon, stationeryIcon, sender, subject, money, CODAmount, daysLeft, hasItem, wasRead, wasReturned, 
				textCreated, canReply, isGM = GetInboxHeaderInfo(index);
			if money > 0 then
				local taxMoney = money*GTRTaxPercentage/100;
				if taxMoney > 0 then
					if GTRQuietMode == 0 then
						print(format("|cffFF7C0AGTR|r: Taxed auction house tax: %s", GetCoinText(taxMoney)));
					end
					GTRTaxDue = GTRTaxDue + taxMoney;
				end
			end
		end
		origTakeInboxMoney(...);
	end
end

local function GTR_UpdateTaxAfterTransaction()
	local newMoney = GetMoney();
	if newMoney > GTRCurrentMoney then
		local taxMoney = (newMoney - GTRCurrentMoney)*GTRTaxPercentage/100;
		GTRTaxDue = GTRTaxDue + taxMoney;
		if taxMoney>0 then
			if GTRQuietMode == 0 then
				print(format("|cffFF7C0AGTR|r: Taxed sales: %s", GetCoinText(taxMoney)));
			end
		end
		-- prevent double taxation in case events fire twice
		GTRCurrentMoney = newMoney;
	end
end

local function GTR_UpdateTaxAfterQuestOrLoot(chatmsg)
	-- parse the message
	local gold = tonumber(chatmsg:match("(%d+)%s" .. GTRGoldStr)) or 0;
	local silver = tonumber(chatmsg:match("(%d+)%s" .. GTRSilverStr)) or 0;
	local copper = tonumber(chatmsg:match("(%d+)%s" .. GTRCopperStr)) or 0;
	local newMoney = gold * COPPER_PER_GOLD + silver * COPPER_PER_SILVER + copper;
	local taxMoney = newMoney*GTRTaxPercentage/100;
	if taxMoney > 0 then
		if GTRQuietMode == 0 then
			print(format("|cffFF7C0AGTR|r: Taxed loot/quest: %s", GetCoinText(taxMoney)));
		end
		GTRTaxDue = GTRTaxDue + taxMoney;
	end
end

local function GTR_UpdateTaxRate()
	local guildInfo = GetGuildInfoText();
	if guildInfo == nil or guildInfo == "" then
		return;
	end
	local newRateText = "nothing";
	local newRate;
	-- first check officer's note for player
	local whoami = UnitName("player");
	local i;
	for i=1, GetNumGuildMembers() do
		local playerName,_,_,_,_,_,_,officerNote = GetGuildRosterInfo(i);
		if playerName == whoami or playerName == whoami.."-"..GetRealmName() then
			if officerNote ~= nil then
				newRateText = officerNote:lower();
			end
			break;
		end
	end
	newRate = newRateText:match("tax%s*[:=]?%s*(%d+)%%");
	if newRate == nil then
		-- not found in officer's note, search GuildInfoText
		newRateText = GetGuildInfoText();
		if newRateText ~= nil then
			newRate = newRateText:lower():match("tax%s*[:=]?%s*(%d+)%%");
		end
	end
	if newRate == nil then
		newRate = 10;
	else
		newRate = tonumber(newRate);
		if newRate > 100 or newRate < 0 then 
			newRate = 10;
		end
	end
	if newRate ~= GTRTaxPercentage then
		GTRTaxPercentage = newRate;
		print(format("|cffFF7C0AGTR|r: Tax Rate: %d%%", GTRTaxPercentage));
	end
end

local function GTR_AuditCommand(command, sender)
	if command:find("!GTR") ~= nil then
		-- only rank 5 or higher should be able to issue that command
		local i;
		local found = 0;
		for i=1, GetNumGuildMembers() do
			local playerName,_,playerRank = GetGuildRosterInfo(i);
			-- player name may be in format name-server
			if playerName == sender or playerName == sender.."-"..GetRealmName() then
				-- this can be optimized to check for specific permissions
				-- of the rank and cache the minimum level at load time
				-- instead of hardcoding rank 5
				found = 1;
				if playerRank < 5 then
					if command == "!GTRaudit" then
						SendChatMessage("GTR Active, Rate: "..GTRTaxPercentage.."%, Tax due: "..GetCoinText(GTRTaxDue)..", Paid to date: "..GetCoinText(GTRTaxToDate),
							"WHISPER", nil, sender);
					elseif command == "!GTRguild" then
						SendChatMessage("GTR Active, Rate: "..GTRTaxPercentage.."%, Tax due: "..GetCoinText(GTRTaxDue)..", Paid to date: "..GetCoinText(GTRTaxToDate),
							"GUILD", nil, nil);
					elseif command == "!GTRupdate" then
						GTR_UpdateTaxRate();
					end
					break;
				else
				end
			end
		end
	end
end

local function GTR_PayTax()
	local toPayTax;
	-- can we even pay the whole amount of the tax?
	if GetMoney() > GTRTaxDue 
	then
		toPayTax = GTRTaxDue;
	-- else
	-- toPayTax = GetMoney();
	end
	-- Pay the tax to the guildbank
	if toPayTax > 0
	then
		DepositGuildBankMoney(toPayTax);
		GTRTaxDue = GTRTaxDue - toPayTax;
		GTRTaxToDate = GTRTaxToDate + toPayTax;
	end
end

function GTR_OnEvent(self, event, ...)
	local arg1, arg2 = ...;
	if event == "ADDON_LOADED" then
		if arg1 == "GTR" then
			-- first time we load the addon, GTRTaxDue/GTRTaxPercentage will be nil
			if GTRTaxDue == nil then
				GTRTaxDue = 0;
			end
			if GTRTaxToDate == nil then
				GTRTaxToDate = 0;
			end
			if GTRTaxPercentage == nil then
				GTRTaxPercentage = 0;
			end
			if GTRQuietMode == nil then
				GTRQuietMode = 0;
			end
			GTRCurrentMoney = GetMoney();
			-- Figure out what GOLD, SILVER, and COPPER are in the client language 
			GTRGoldStr = strmatch(format(GOLD_AMOUNT, 20), "%d+%s(.+)") -- This will return what's in the brackets, which on enEU would be "Gold"
			GTRSilverStr = strmatch(format(SILVER_AMOUNT, 20), "%d+%s(.+)")
			GTRCopperStr = strmatch(format(COPPER_AMOUNT, 20), "%d+%s(.+)")
			-- print welcome message
			print("|cffFF7C0AGuild Tax Rate|r version "..GetAddOnMetadata("GTR","Version")..". Type |cffFF7C0A/GTR help|r for help."); 
		end
	elseif event == "PLAYER_ENTERING_WORLD" then
		GTR_UpdateTaxRate();	-- pull guild info on entering world
	elseif event == "GUILD_ROSTER_UPDATE" then
		GTR_UpdateTaxRate();
	elseif event == "CHAT_MSG_GUILD" or event == "CHAT_MSG_WHISPER" then
		GTR_AuditCommand(arg1, arg2);
	elseif event == "MERCHANT_SHOW" then
		GTRCurrentMoney = GetMoney();
	elseif event == "MERCHANT_CLOSED" then
		GTR_UpdateTaxAfterTransaction();
	elseif event == "CHAT_MSG_MONEY" or event == "CHAT_MSG_SYSTEM" then
		GTR_UpdateTaxAfterQuestOrLoot(arg1);
	elseif event == "GUILDBANKBAGSLOTS_CHANGED" then
		GTR_PayTax();
	end
end
